/*
 * =====================================================================================
 *
 *       Filename:  configuration.c
 *
 *    Description:  Basic configuration file reader
 *
 *        Version:  1.0
 *        Created:  29.03.2014 21:03:54
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Lucjan Bryndza (LB), lucck(at)boff(dot)pl
 *   Organization:  BoFF
 *
 * =====================================================================================
 */
#include "configuration.h"
#include "keypad.h"
#include "systick.h"
#include "display.h"
#include <avr/eeprom.h>
#include <stdlib.h>
#include <string.h>
/* ------------------------------------------------------------------ */ 
#define IFF_MAX  99999999L
#define IFF_MIN -99999999L
#define VFAIL_MIN 7000
#define VFAIL_MAX 19000
/* ------------------------------------------------------------------ */
//! EEPROM IF frequency config value
static EEMEM uint32_t ee_iff_value = 0;
//! EEPROM poweroff treshold value
static EEMEM uint16_t ee_vfail_value = 9500;
/* ------------------------------------------------------------------ */ 
typedef enum cfg_state_e {
	cfg_MHz,	//! Configure IFF
	cfg_KHz,
	cfg_Hz,
	cfg_Vf		//! Configure fail voltage
} cfg_state;
/* ------------------------------------------------------------------ */
//! Read iff frequency 
config_if_t config_get_if_freq( void )
{
	union {
		config_if_t s; 
		uint32_t u;
	} iff;
	iff.u = eeprom_read_dword( &ee_iff_value );
	if( iff.s <= IFF_MAX && iff.s >= IFF_MIN ) {
		return iff.s;
	} else {
		return 0;
	}
}
/* ------------------------------------------------------------------ */ 
//! Get voltage level value
config_vfail_t config_get_vfail( void )
{
	config_vfail_t v = eeprom_read_word( &ee_vfail_value );
	if( v >= VFAIL_MIN && v <= VFAIL_MAX ) {
		return v;
	} else {
		return 0;
	}
}
/* ------------------------------------------------------------------ */ 
//! Set if freq
void config_setup( void ) 
{
	displ_blink( true );
	config_if_t f = config_get_if_freq();
	config_vfail_t vfail = config_get_vfail();
	//Configuration table fill
	int16_t ptab[4];
	char buf[16];
	ptab[cfg_MHz] = f / 1000000L;
	f = labs(f) % 1000000L;
	ptab[cfg_Hz] = f % 1000;
	f /= 1000;
	ptab[cfg_KHz]  = f % 1000;
	//! Configure voltage
	ptab[cfg_Vf] = vfail / 100;
	//! Display first item
	uint8_t pos = 0;
	itoa( ptab[pos], buf, 10 );
	strcat( buf, "M" );
	displ_string( buf, ' ' );
	for(;;) {
		keyb_t key = keyb_get();
		if ( key == (key_right|key_rpt) ) {
			// Configure and write IF freq
			bool negative = ( ptab[cfg_MHz] < 0 );
			ptab[cfg_MHz] = labs( ptab[cfg_MHz] );
			f = (int32_t)ptab[cfg_MHz] * 1000000L + (int32_t)ptab[cfg_KHz] * 1000L + (int32_t)ptab[cfg_Hz];
			if( negative ) f = -f;
			eeprom_write_dword( &ee_iff_value, f );
			// Configure and write voltage freq
			eeprom_write_word( &ee_vfail_value, ptab[cfg_Vf] * 100 );
			displ_blink( false );
			displ_string("ok", ' ' );
			while( kbhit() ) {
				wdt_reset();
				sleep_mode();
			}
			break;
		} else if( key == key_right ) {
			if( ++pos > cfg_Vf ) {
				pos = cfg_MHz;
			}
			itoa( ptab[pos], buf, 10 );
			if( pos == cfg_MHz ) {
				strcat( buf, "M" );
			} else if( pos == cfg_KHz ) {
				strcat( buf, "k" );
			} else if( pos == cfg_Hz ) {
				strcat( buf, "H" );
			} else if( pos == cfg_Vf ) {
				strcat( buf, "V" );
			}
			displ_string( buf, ' ' );
			if( pos == cfg_Vf ) {
				displ_set_dot( 0, true );
			}
		} else if( key & key_left ) {
			if( key & key_rpt ) {
				ptab[pos]+=10;
			} else {
				ptab[pos]+=1;
			}
			if( pos == cfg_MHz ) {
				if(ptab[pos] > 99 ) {
					ptab[pos] = -99;
				}
			} else if (pos == cfg_Vf ) {
				if( ptab[pos] > VFAIL_MAX/100 ) {
					ptab[pos] = VFAIL_MIN/100;
				}
			} else {
				if( ptab[pos] > 999 ) {
					ptab[pos] = 0;
				}
			}
			itoa( ptab[pos], buf, 10 );
			displ_string( buf, ' ' );
			if( pos == cfg_Vf ) {
				displ_set_dot( 2, true );
			}
		} 		
		//Reset watchdog
		wdt_reset();
		sleep_mode();
	}
}
/* ------------------------------------------------------------------ */ 
