/*
 * =====================================================================================
 *
 *       Filename:  led_display.c
 *
 *    Description:  LED display avr implementation
 *
 *        Version:  1.0
 *        Created:  28.03.2014 19:56:01
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Lucjan Bryndza (LB), lucck(at)boff(dot)pl
 *   Organization:  BoFF
 *
 * =====================================================================================
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/atomic.h>
#include <string.h>
#include "display.h"
#include "voltmeter.h"
#include "keypad.h"
#include "systick.h"
/* ------------------------------------------------------------------ */
#define TIMER_PRELOAD ((((unsigned long)F_CPU)/256UL)/SYSTEM_HZ)
//! Display blinking period
#define DISP_BLINK_PERIOD 4
//PIN special definition
#define DISP_PORT_MASK_1 (_BV(PB0)|_BV(PB1)|_BV(PB2)|_BV(PB4)|_BV(PB5) )
#define DISP_PORT_1 PORTB
#define DISP_PORT_MASK_2 (_BV(PD0)|_BV(PD1)|_BV(PD4))
#define DISP_PORT_2 PORTD
#define DISP_MUX_PORT PORTC
#define DISP_MUX_MASK (_BV(PC2)|_BV(PC3)|_BV(PC4)|_BV(PC5))
#define DISP_DDR_1 DDRB
#define DISP_DDR_2 DDRD
#define DISP_DDR_MUX DDRC
/* ------------------------------------------------------------------ */
//! Voltmeter time interrupt
void _voltmeter_time_interrupt( void );
//! Keyb time interrupt
void _keyb_time_interrupt(void);
/* ------------------------------------------------------------------ */
//! Display buffer data
static volatile uint8_t disp_buf[4];
//! Blink flag
static volatile bool disp_blink;
//! Tick timer
static volatile tick_t sys_tick;
/* ------------------------------------------------------------------ */
/** Set segment port bit1 segment .,G to A */
static void set_segment_port( uint8_t value )
{
	value = ~value;
	DISP_PORT_1 &= ~DISP_PORT_MASK_1;
	DISP_PORT_1 |= (value & 0x07) | (((value>>3)&0x3)<<4 );
	DISP_PORT_2 &= ~DISP_PORT_MASK_2;
	DISP_PORT_2 |= ((value>>5)&0x3)|((value>>7)<<4);
}
/* ------------------------------------------------------------------ */
static uint8_t ascii_seg_translate( char ch )
{
	switch( ch ) {
		case '0': return 0b11111100;
		case '1': return 0b01100000;
		case '2': return 0b11011010;
		case '3': return 0b11110010;
		case '4': return 0b01100110;
		case '5': return 0b10110110;
		case '6': return 0b10111110;
		case '7': return 0b11100000;
		case '8': return 0b11111110;
		case '9': return 0b11110110;
		case 'A': case 'a': return 0b11101110;
		case 'B': case 'b': return 0b00111110;
		case 'c': case 'C': return 0b10011100;
		case 'D': case 'd': return 0b01111010;
		case 'E': case 'e': return 0b10011110;
		case 'F': case 'f': return 0b10001110;
		case 'V': case 'v': return 0b01111100;
		case 'o': case 'O': return 0b00111010;
		case 'k': case 'K': return 0b00001110;
		case 'h': case 'H': return 0b01101110;
		case 'g': case 'G': return 0b11011110;
		case 'n': case 'N': case 'm': case 'M': return 0b00101010;
		case '-': return 0b00000010;
		default: return 0;
	}
}
/* ------------------------------------------------------------------ */ 
//Display init implementation 
void displ_init(void) {
	//Configure port
	DISP_DDR_1 |= DISP_PORT_MASK_1;
	DISP_DDR_2 |= DISP_PORT_MASK_2;
	DISP_DDR_MUX |= DISP_MUX_MASK;
	DISP_MUX_PORT |= DISP_MUX_MASK;
	//Initialize timer 2 as pinpad
	TCCR0 = _BV(CS02);
	TIMSK |= _BV(TOIE0); 
}
/* ------------------------------------------------------------------ */
//Display deinitialization
void displ_deinit(void) 
{
	TIMSK &= ~_BV(TOIE0);
	TCCR0 = 0;
	ATOMIC_BLOCK( ATOMIC_RESTORESTATE ) {
		DISP_DDR_1 &= ~DISP_PORT_MASK_1;
		DISP_DDR_2 &= ~DISP_PORT_MASK_2;
		DISP_DDR_MUX &= ~DISP_MUX_MASK;
		DISP_PORT_1 &= ~DISP_PORT_MASK_1;
		DISP_PORT_2 &= ~DISP_PORT_MASK_2;
		DISP_MUX_PORT &= ~DISP_MUX_MASK;
	}
}
/* ------------------------------------------------------------------ */
//! Display char
void displ_char( char ch, uint8_t pos ) 
{
	disp_buf[pos] = ascii_seg_translate( ch );
}
/* ------------------------------------------------------------------ */ 
//! Display string from PGMP space
void displ_string( const char* str, char rpad_char )
{
	uint8_t slen = strlen( str );
	if( slen > sizeof disp_buf ) {
		slen = sizeof disp_buf;
	}
	uint8_t p;
	if( rpad_char ) {
		uint8_t clr_pos = sizeof(disp_buf)-slen;
		for( p = 0; p < clr_pos; ++p ) {
			displ_char( rpad_char , p );
		}
	} else {
		p = 0;
	}
	for(uint8_t dp=0; dp<slen; ++p,++dp ) {
		displ_char( str[dp], p);
	}
}
/* ------------------------------------------------------------------ */ 
//! Set dot on selected position
void displ_set_dot( uint8_t pos, bool en )
{
	if( pos >= sizeof disp_buf ) {
		return;
	}
	if( en ) {
		disp_buf[ pos ] |= 1;
	}
}
/* ------------------------------------------------------------------ */ 
//!Enable display blinking
void displ_blink( bool en )
{
	disp_blink = en;
}
/* ------------------------------------------------------------------ */ 
ISR(  TIMER0_OVF_vect , ISR_NOBLOCK ) 
{
	static uint8_t port_mask = _BV(PC2);
	static uint8_t buf_pos = 0;
	static uint16_t blink_cnt = 0;
	static bool blink_on = true;
	static uint8_t adc_psc = 0;
	static uint8_t keyb_psc = 0;
	TCNT0 += 256-TIMER_PRELOAD;
	DISP_MUX_PORT |= DISP_MUX_MASK;
	if( disp_blink ) {
		if(++blink_cnt >= SYSTEM_HZ/DISP_BLINK_PERIOD ) {
			blink_cnt = 0;
			blink_on = !blink_on;
		}
		if( !blink_on ) {
			return;
		}
	}
	DISP_MUX_PORT &= ~port_mask; 
	set_segment_port( disp_buf[ buf_pos ] );
	port_mask <<= 1;
	if( ++buf_pos >= sizeof disp_buf ) {
		port_mask = _BV(PC2);
		buf_pos = 0;
	}
	if( ++adc_psc == SYSTEM_HZ/VMTR_ADC_HZ ) {
		adc_psc = 0;
		_voltmeter_time_interrupt();
	}
	if( ++keyb_psc == SYSTEM_HZ/KEYB_HANDLER_HZ ) {
		keyb_psc = 0;
		_keyb_time_interrupt();
	}
	++sys_tick;
}
/* ------------------------------------------------------------------ */
//! Read system tick
tick_t read_tick(void) 
{
	tick_t t1,t2;
	do {
		t1 = sys_tick;
		t2 = sys_tick;
	} while( t1 != t2 );
	return t1;
}
/* ------------------------------------------------------------------ */ 
//! Clear display
void displ_clear(void)
{
	memset( (uint8_t*)disp_buf, 0, sizeof disp_buf );
}
/* ------------------------------------------------------------------ */ 
