/*
 * =====================================================================================
 *
 *       Filename:  voltmeter.c
 *
 *    Description:  Voltmeter implementation for AVR
 *
 *        Version:  1.0
 *        Created:  29.03.2014 14:52:29
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Lucjan Bryndza (LB), lucck(at)boff(dot)pl
 *   Organization:  BoFF
 *
 * =====================================================================================
 */
#include <avr/io.h>
#include <avr/interrupt.h>
#include "voltmeter.h"
/* ------------------------------------------------------------------ */
#define AGC_NUM_SAMPLES 8	//! AGC num samples
#define VCC_NUM_SAMPLES 8	//! VCC num samples
#define AGC_SAMPLE_TIME VMTR_ADC_HZ/4	//!AGC sample
#define ADC_VCC_MUX 0			//!VCC MUX
#define ADC_AGC_MUX (_BV(MUX0)) //! AGC Mux
#define ADC_MASK_MUX (_BV(MUX3)|_BV(MUX2)|_BV(MUX1)|_BV(MUX0))
/* ------------------------------------------------------------------ */
#define R1_VAL 6800UL	//! Voltage divider
#define R2_VAL 1000UL	//! Voltage divider
#define ADC_MAX 1024UL
#define V_REF 2560UL
/* ------------------------------------------------------------------ */
static volatile vmtr_t g_vcc[VCC_NUM_SAMPLES]; //! Voltage value
static volatile vmtr_t g_agc[AGC_NUM_SAMPLES]; //! AGC values
volatile static uint8_t g_curr_mux;
volatile static uint8_t g_agc_pos;
volatile static uint8_t g_vcc_pos;
/* ------------------------------------------------------------------ */
//! Deinitialize the voltage meter
void vmtr_deinit( void ) 
{
	ADCSRA = 0;
	ADMUX = 0;
}
/* ------------------------------------------------------------------ */
//! Voltage measure 
void vmtr_init( void )
{
	ADCSRA = _BV(ADEN)|_BV(ADPS2)|_BV(ADPS1)|_BV(ADIE);
	//Set reference voltage to 2.56
	ADMUX = _BV(REFS1)|_BV(REFS0);
}
/* ------------------------------------------------------------------ */ 
//! Start conversion
static inline void start_conv( uint8_t mux ) 
{
	ADMUX &= ~ADC_MASK_MUX;
	ADMUX |= (mux & ADC_MASK_MUX);
	ADCSRA |= _BV(ADSC);	
	g_curr_mux = mux;
}
/* ------------------------------------------------------------------ */ 
//! Get single value from buffer (atomic)
static vmtr_t get_vcc_pos( uint8_t pos ) 
{
	vmtr_t v1, v2;
	do {
		v1 = g_vcc[pos];
		v2 = g_vcc[pos];
	}
	while( v1!=v2 );
	return v1;
}
/* ------------------------------------------------------------------ */ 
//! Get single value from buffer (atomic)
static vmtr_t get_agc_pos( uint8_t pos ) 
{
	vmtr_t v1, v2;
	do {
		v1 = g_agc[pos];
		v2 = g_agc[pos];
	}
	while( v1!=v2 );
	return v1;
}
/* ------------------------------------------------------------------ */ 
//! Get VCC
vmtr_t vmtr_get_vcc( void )
{
	uint32_t vcc = 0;
	for( uint8_t i=0; i<VCC_NUM_SAMPLES; ++i ) {
		vcc += get_vcc_pos( i );
	}
	vcc /=  VCC_NUM_SAMPLES;
	vcc = ( vcc * V_REF ) / ADC_MAX;
	vcc = (vcc * ( R1_VAL + R2_VAL ))/R2_VAL;
	return vcc;
}
/* ------------------------------------------------------------------ */ 
//! Get AGC
vmtr_t vmtr_get_agc( void )
{
	uint32_t agc = 0;
	for( uint8_t i=0; i<AGC_NUM_SAMPLES; ++i ) {
		agc += get_agc_pos( i );
	}
	agc /= AGC_NUM_SAMPLES;
	agc = ( agc * V_REF ) / ADC_MAX;
	return agc;
}
/* ------------------------------------------------------------------ */ 
//! Voltmeter time interrupt
void _voltmeter_time_interrupt( void )
{
	static uint8_t agc_psc = 0;
	if( ++agc_psc ==  AGC_SAMPLE_TIME ) {
		start_conv( ADC_AGC_MUX );
		agc_psc = 0;
	} else {
		start_conv( ADC_VCC_MUX );
	}
}
/* ------------------------------------------------------------------ */ 
//! ADC completion ISR vector
ISR( ADC_vect, ISR_NOBLOCK )
{
	if( g_curr_mux == ADC_AGC_MUX ) {
		g_agc[ g_agc_pos ] = ADC;
		if( ++g_agc_pos == AGC_NUM_SAMPLES ) {
			g_agc_pos = 0;
		}
	} else if( g_curr_mux == ADC_VCC_MUX ) {
		g_vcc[ g_vcc_pos ] = ADC;
		if( ++g_vcc_pos == VCC_NUM_SAMPLES ) {
			g_vcc_pos = 0;
		}
	}
}
/* ------------------------------------------------------------------ */ 
