/*
 * =====================================================================================
 *
 *       Filename:  vufmeter.c
 *
 *    Description:  Taurus VUF meter core file
 *
 *        Version:  1.0
 *        Created:  28.03.2014 19:35:43
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Lucjan Bryndza (LB), lucck(at)boff(dot)pl
 *   Organization:  BoFF
 *
 * =====================================================================================
 */
#include "display.h"
#include "fmeas.h"
#include "voltmeter.h"
#include "keypad.h"
#include "systick.h"
#include "configuration.h"
#include "poweroff.h"
#include "config.h"
#include <string.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <stdlib.h>
/* ------------------------------------------------------------------ */
typedef enum app_state_e {
	disp_unknown,	//! Unknown state
	disp_freq, 		//! Display frequency
	disp_agc,		//! Display AGC
	disp_volt		//! Display voltage
} app_state;
/* ------------------------------------------------------------------ */
//! Application context
typedef struct app_ctx_s {
	app_state state;		//! Current display state
	app_state p_state;		//! Previous display state
	tick_t disp_time;		//! Display time timer
	tick_t f_timer;		    //! MHz show timer
	bool   show_mhz; 		//! Currently show mhz
#if CONFIG_HAVE_POWER_CTL_FEATURE
	tick_t valid_v_fail_time;	//! Last valid voltage time
#endif
}  app_ctx;
/* ------------------------------------------------------------------ */ 
//! Introduce splash screen
static void splash_screen( void )
{
	displ_string("F9B", 0 );
	timer_wait( SYSTEM_HZ );
	displ_clear();
}
/* ------------------------------------------------------------------ */ 
//!Display frequency
static void display_frequency( app_ctx* ctx )
{
	config_if_t iff = config_get_if_freq();
	fmeas_t f = fmeas_get_value();
	f = (iff>=0)?( iff + f ):( -iff - f );
	if( f >= 99999999UL ) {
		displ_string( "badF", 0 );
		ctx->f_timer = 0;
		ctx->show_mhz = false;
		return;
	}
	//Frequency in 0.1khz unit
	f /= 100;
	uint8_t fmhz = f / 10000UL;
	if( fmhz > 0 )  {
		if( ctx->show_mhz ) {
			if( timer_elapsed( ctx->f_timer, SYSTEM_HZ ) ) {
				ctx->show_mhz = false;
				ctx->f_timer = read_tick();
			}
		} else {
			if( timer_elapsed( ctx->f_timer, SYSTEM_HZ * 5 ) ) {
				ctx->show_mhz = true;
				ctx->f_timer = read_tick();
			}
		}
	} else {
		ctx->f_timer = 0;
		ctx->show_mhz = false;
	}
	char buf[16];
	if( ctx->show_mhz ) {		//Show MHz major
		utoa( fmhz, buf, 10 );
		strcat( buf, "f" );
		displ_string( buf, ' ' );
	} else {					//Show MHz minor
		ultoa( f%10000, buf, 10 );
		displ_string( buf, '0' );
		displ_set_dot( 2, true );
	}
}
/* ------------------------------------------------------------------ */
//! Display voltage
static void display_voltage( void )
{
	vmtr_t v = vmtr_get_vcc();
	v = v / 100 + ( v % 100 > 50 );
	char buf[16];
	utoa( v, buf, 10 );
	strcat( buf, "V" );
	displ_string( buf, ' ' );
	displ_set_dot( 0, true );
}
/* ------------------------------------------------------------------ */
//! Display smeter
static void display_smeter(void)
{
	vmtr_t v = vmtr_get_agc();
	v = v / 10 + ( v % 10 > 5 );
	char buf[16] = { 'o' };
	utoa( v, buf, 10 );
	strcat(buf, "o");
	displ_string( buf, ' ' );
}
/* ------------------------------------------------------------------ */ 
#if CONFIG_HAVE_POWER_CTL_FEATURE
//! Voltage monitor input
static void voltage_monitor( app_ctx* ctx )
{
	vmtr_t vfail = config_get_vfail();
	if( vfail == 0 ) {	//Check voltage not needed
		return;
	}
	vmtr_t v = vmtr_get_vcc();
	if( v > vfail ) {
		ctx->valid_v_fail_time = read_tick();
	}
	if(  timer_elapsed(ctx->valid_v_fail_time,
			(tick_t)CONFIG_VOLTAGE_POWEROFF_TOUT*(tick_t)SYSTEM_HZ )
	 ) {
		displ_string("Voff", 0 );
		timer_wait( SYSTEM_HZ *2 );
		power_off();
	}
}
#endif
/* ------------------------------------------------------------------ */ 
//! Function main
int main (void)
{
	//Set basic sleep mode
	set_sleep_mode( SLEEP_MODE_IDLE );
	//Set watchdog
	wdt_enable( WDTO_250MS );
	//! Display initialization
	displ_init();
	//! Frequency measure initialization
	fmeas_init();
	//! Voltage meter initialization
	vmtr_init();
	//! Keyboard initialization
	keyb_init();
	//! Enable interrupt
	sei();
	//! Intro initialization
	splash_screen();
#if CONFIG_HAVE_POWER_CTL_FEATURE
	//! Power on rest of the TRX
	power_on();
#endif
	//! Check if config  IF freq state is required
	while( kbhit() ) {
		if( keyb_get() == (key_right|key_rpt) ) {
			displ_string( "cfg ", 0 );
			while( kbhit() ) {
				wdt_reset();
				sleep_mode();
			}
			//! Call the setup configuration
			keyb_get();
			config_setup();
			keyb_get();
		}
		wdt_reset();
		sleep_mode();
	}
	//! Application state
	app_ctx ctx;
	ctx.state = disp_freq;
	ctx.p_state = disp_unknown;
	ctx.disp_time = read_tick();
	ctx.show_mhz = false;
	ctx.f_timer = read_tick();
#if CONFIG_HAVE_POWER_CTL_FEATURE
	ctx.valid_v_fail_time = read_tick();
#endif
	keyb_get();
	//! Main loop
	for(;;) {
		keyb_t key = keyb_get();
		if( key == key_right ) {
			if(++ctx.state > disp_volt ) {
				ctx.state = disp_freq;
			}
		} 
#if CONFIG_HAVE_POWER_CTL_FEATURE
		else if( key == (key_left|key_rpt) ) {
			displ_string("off ", 0 );
			while( kbhit() ) {
				wdt_reset();
				sleep_mode();
			}
			power_off();
			return 0;
		}
#endif
#if CONFIG_HAVE_POWER_CTL_FEATURE
		//Voltage monitor
		voltage_monitor( &ctx );
#endif
		//Enable or disable frequency counting
		if( ctx.state == disp_freq && ctx.p_state != disp_freq ) {
			fmeas_start();
			ctx.show_mhz = false;
			ctx.f_timer = read_tick();
		} else if( ctx.p_state == disp_freq && ctx.state != disp_freq ) {
			fmeas_stop();
		}
		//! Display timer timeout
		while( timer_elapsed(ctx.disp_time, SYSTEM_HZ/5 ) ) {
			switch( ctx.state ) {
				case disp_freq: display_frequency( &ctx ); break;
				case disp_volt: display_voltage(); break;
				case disp_agc:  display_smeter(); break;
				default: break;
			}
			ctx.disp_time = read_tick();
		}
		ctx.p_state = ctx.state;
		wdt_reset();
		sleep_mode();
		
	}
}
/* ------------------------------------------------------------------ */
